@@ -77,8 +77,6 @@ pub enum PopulationType {
         max_node_size: Option<usize>,
         /// Spread factor. Default is 0.25.
         spread_factor: Option<f64>,
-        /// The reduction factor. Default is 0.1.
-        reduction_factor: Option<f64>,
         /// Distribution factor. Default is 0.25.
         distribution_factor: Option<f64>,
         /// Learning rate. Default is 0.1.
@@ -122,6 +120,11 @@ pub enum HyperType {
         /// A collection of inner mutation operators (metaheuristics).
         mutations: Vec<MutationType>,
     },
+
+    /// A hyper heuristic which selects mutations from the predefined list using reinforcement
+    /// learning technics.
+    #[serde(rename(deserialize = "dynamic-selective"))]
+    DynamicSelective,
 }
 
 /// A mutation operator configuration.
@@ -386,29 +389,29 @@ impl Default for Config {
 fn configure_from_evolution(
     mut builder: Builder,
     population_config: &Option<EvolutionConfig>,
-    problem: Arc<Problem>,
-    environment: Arc<Environment>,
 ) -> Result<Builder, String> {
     if let Some(config) = population_config {
         if let Some(initial) = &config.initial {
+            let environment = builder.config.environment.clone();
             builder = builder.with_init_params(
                 initial.size,
-                initial
-                    .methods
-                    .as_ref()
-                    .map(|methods| methods.iter().map(|method| create_recreate_method(method)).collect()),
+                initial.methods.as_ref().map(|methods| {
+                    methods.iter().map(|method| create_recreate_method(method, environment.clone())).collect()
+                }),
             );
         }
 
         if let Some(variation) = &config.population {
-            let default_selection_size = get_default_selection_size(environment.as_ref());
+            let default_selection_size = get_default_selection_size(builder.config.environment.as_ref());
             let population = match &variation {
-                PopulationType::Greedy { selection_size } => {
-                    Box::new(Greedy::new(problem, selection_size.unwrap_or(default_selection_size), None))
-                }
+                PopulationType::Greedy { selection_size } => Box::new(Greedy::new(
+                    builder.config.problem.clone(),
+                    selection_size.unwrap_or(default_selection_size),
+                    None,
+                )),
                 PopulationType::Elitism { max_size, selection_size } => Box::new(Elitism::new(
-                    problem,
-                    environment.random.clone(),
+                    builder.config.problem.clone(),
+                    builder.config.environment.random.clone(),
                     max_size.unwrap_or(4),
                     selection_size.unwrap_or(default_selection_size),
                 ))
@@ -417,7 +420,6 @@ fn configure_from_evolution(
                     max_elite_size,
                     max_node_size,
                     spread_factor,
-                    reduction_factor,
                     distribution_factor,
                     learning_rate,
                     selection_size,
@@ -438,9 +440,6 @@ fn configure_from_evolution(
                     if let Some(spread_factor) = spread_factor {
                         config.spread_factor = *spread_factor;
                     }
-                    if let Some(reduction_factor) = reduction_factor {
-                        config.reduction_factor = *reduction_factor;
-                    }
                     if let Some(distribution_factor) = distribution_factor {
                         config.distribution_factor = *distribution_factor;
                     }
@@ -457,7 +456,7 @@ fn configure_from_evolution(
                         config.exploration_ratio = *exploration_ratio;
                     }
 
-                    Box::new(Rosomaxa::new(problem, environment, config)?)
+                    Box::new(Rosomaxa::new(builder.config.problem.clone(), builder.config.environment.clone(), config)?)
                 }
             };
 
@@ -468,21 +467,26 @@ fn configure_from_evolution(
     Ok(builder)
 }
 
-fn configure_from_hyper(
-    mut builder: Builder,
-    hyper_config: &Option<HyperType>,
-    environment: Arc<Environment>,
-) -> Result<Builder, String> {
+fn configure_from_hyper(mut builder: Builder, hyper_config: &Option<HyperType>) -> Result<Builder, String> {
     if let Some(config) = hyper_config {
         match config {
             HyperType::StaticSelective { mutations } => {
                 let mutation_group = mutations
                     .iter()
-                    .map(|mutation| create_mutation(&builder.config.problem, environment.random.clone(), mutation))
+                    .map(|mutation| {
+                        create_mutation(&builder.config.problem, builder.config.environment.clone(), mutation)
+                    })
                     .collect::<Result<Vec<_>, _>>()?;
                 let static_selective = vrp_core::solver::hyper::StaticSelective::new(mutation_group);
                 builder = builder.with_hyper(Box::new(static_selective));
             }
+            HyperType::DynamicSelective => {
+                let dynamic_selective = vrp_core::solver::hyper::DynamicSelective::new_with_defaults(
+                    builder.config.problem.clone(),
+                    builder.config.environment.clone(),
+                );
+                builder = builder.with_hyper(Box::new(dynamic_selective));
+            }
         }
     }
 
@@ -499,36 +503,45 @@ fn configure_from_termination(mut builder: Builder, termination_config: &Option<
     builder
 }
 
-fn create_recreate_method(method: &RecreateMethod) -> (Arc<dyn Recreate + Send + Sync>, usize) {
+fn create_recreate_method(
+    method: &RecreateMethod,
+    environment: Arc<Environment>,
+) -> (Arc<dyn Recreate + Send + Sync>, usize) {
     match method {
         RecreateMethod::Cheapest { weight } => (Arc::new(RecreateWithCheapest::default()), *weight),
         RecreateMethod::Farthest { weight } => (Arc::new(RecreateWithFarthest::default()), *weight),
         RecreateMethod::SkipBest { weight, start, end } => (Arc::new(RecreateWithSkipBest::new(*start, *end)), *weight),
-        RecreateMethod::Blinks { weight } => (Arc::new(RecreateWithBlinks::<SingleDimLoad>::default()), *weight),
+        RecreateMethod::Blinks { weight } => {
+            (Arc::new(RecreateWithBlinks::<SingleDimLoad>::new_with_defaults(environment.random.clone())), *weight)
+        }
         RecreateMethod::Gaps { weight, min } => (Arc::new(RecreateWithGaps::new(*min)), *weight),
         RecreateMethod::Nearest { weight } => (Arc::new(RecreateWithNearestNeighbor::default()), *weight),
         RecreateMethod::Regret { weight, start, end } => (Arc::new(RecreateWithRegret::new(*start, *end)), *weight),
         RecreateMethod::Perturbation { weight, probability, min, max } => {
-            (Arc::new(RecreateWithPerturbation::new(*probability, *min, *max)), *weight)
+            (Arc::new(RecreateWithPerturbation::new(*probability, *min, *max, environment.random.clone())), *weight)
         }
     }
 }
 
 fn create_mutation(
     problem: &Arc<Problem>,
-    random: Arc<dyn Random + Send + Sync>,
+    environment: Arc<Environment>,
     mutation: &MutationType,
 ) -> Result<(Arc<dyn Mutation + Send + Sync>, MutationProbability), String> {
     Ok(match mutation {
         MutationType::RuinRecreate { probability, ruins, recreates } => {
             let ruin = Arc::new(WeightedRuin::new(ruins.iter().map(|g| create_ruin_group(problem, g)).collect()));
-            let recreate =
-                Arc::new(WeightedRecreate::new(recreates.iter().map(|r| create_recreate_method(r)).collect()));
-            (Arc::new(RuinAndRecreate::new(ruin, recreate)), create_mutation_probability(probability, random.clone()))
+            let recreate = Arc::new(WeightedRecreate::new(
+                recreates.iter().map(|r| create_recreate_method(r, environment.clone())).collect(),
+            ));
+            (
+                Arc::new(RuinAndRecreate::new(ruin, recreate)),
+                create_mutation_probability(probability, environment.random.clone()),
+            )
         }
         MutationType::LocalSearch { probability, times, operators: inners } => {
             let operator = create_local_search(times, inners);
-            (Arc::new(LocalSearch::new(operator)), create_mutation_probability(probability, random.clone()))
+            (Arc::new(LocalSearch::new(operator)), create_mutation_probability(probability, environment.random.clone()))
         }
         MutationType::Decomposition { routes, repeat, probability } => {
             if *repeat < 1 {
@@ -538,10 +551,11 @@ fn create_mutation(
                 return Err(format!("min routes must be greater than 2. Specified: {}", routes.min));
             }
 
-            let mutation = vrp_core::solver::hyper::StaticSelective::create_default_mutation(problem.clone());
+            let mutation =
+                vrp_core::solver::hyper::StaticSelective::create_default_mutation(problem.clone(), environment.clone());
             (
                 Arc::new(DecomposeSearch::new(mutation, (routes.min, routes.max), *repeat)),
-                create_mutation_probability(probability, random.clone()),
+                create_mutation_probability(probability, environment.random.clone()),
             )
         }
     })
@@ -579,19 +593,19 @@ fn create_ruin_method(problem: &Arc<Problem>, method: &RuinMethod) -> (Arc<dyn R
             (Arc::new(AdjustedStringRemoval::new(*lmax, *cavg, *alpha)), *probability)
         }
         RuinMethod::Neighbour { probability, min, max, threshold } => {
-            (Arc::new(NeighbourRemoval::new(JobRemovalLimit::new(*min, *max, *threshold))), *probability)
+            (Arc::new(NeighbourRemoval::new(RuinLimits::new(*min, *max, *threshold, 8))), *probability)
         }
         RuinMethod::RandomJob { probability, min, max, threshold } => {
-            (Arc::new(RandomJobRemoval::new(JobRemovalLimit::new(*min, *max, *threshold))), *probability)
+            (Arc::new(RandomJobRemoval::new(RuinLimits::new(*min, *max, *threshold, 8))), *probability)
         }
         RuinMethod::RandomRoute { probability, min, max, threshold } => {
             (Arc::new(RandomRouteRemoval::new(*min, *max, *threshold)), *probability)
         }
         RuinMethod::WorstJob { probability, min, max, threshold, skip: worst_skip } => {
-            (Arc::new(WorstJobRemoval::new(*worst_skip, JobRemovalLimit::new(*min, *max, *threshold))), *probability)
+            (Arc::new(WorstJobRemoval::new(*worst_skip, RuinLimits::new(*min, *max, *threshold, 8))), *probability)
         }
         RuinMethod::Cluster { probability, min, max, threshold, cmin, cmax } => (
-            Arc::new(ClusterRemoval::new(problem.clone(), *cmin..*cmax, JobRemovalLimit::new(*min, *max, *threshold))),
+            Arc::new(ClusterRemoval::new(problem.clone(), *cmin..*cmax, RuinLimits::new(*min, *max, *threshold, 8))),
             *probability,
         ),
         RuinMethod::CloseRoute { probability } => (Arc::new(CloseRouteRemoval::default()), *probability),
@@ -691,11 +705,11 @@ pub fn create_builder_from_config_file<R: Read>(
 /// Creates a solver `Builder` from config.
 pub fn create_builder_from_config(problem: Arc<Problem>, config: &Config) -> Result<Builder, String> {
     let environment = configure_from_environment(&config.environment);
-    let mut builder = Builder::new(problem.clone(), environment.clone());
+    let mut builder = Builder::new(problem, environment);
 
     builder = configure_from_telemetry(builder, &config.telemetry);
-    builder = configure_from_evolution(builder, &config.evolution, problem, environment.clone())?;
-    builder = configure_from_hyper(builder, &config.hyper, environment)?;
+    builder = configure_from_evolution(builder, &config.evolution)?;
+    builder = configure_from_hyper(builder, &config.hyper)?;
     builder = configure_from_termination(builder, &config.termination);
 
     Ok(builder)
