@@ -30,22 +30,20 @@ fn can_read_full_config() {
             max_elite_size,
             max_node_size,
             spread_factor,
-            reduction_factor,
             distribution_factor,
             learning_rate,
             rebalance_memory,
             rebalance_count,
             exploration_ratio,
         } => {
-            assert_eq!(selection_size, Some(12));
+            assert_eq!(selection_size, Some(8));
             assert_eq!(max_elite_size, Some(2));
             assert_eq!(max_node_size, Some(2));
             assert_eq!(spread_factor, Some(0.25));
-            assert_eq!(reduction_factor, Some(0.1));
             assert_eq!(distribution_factor, Some(0.25));
             assert_eq!(learning_rate, Some(0.1));
             assert_eq!(rebalance_memory, Some(500));
-            assert_eq!(rebalance_count, Some(4));
+            assert_eq!(rebalance_count, Some(2));
             assert_eq!(exploration_ratio, Some(0.9));
         }
         _ => unreachable!(),
@@ -99,6 +97,7 @@ fn can_read_full_config() {
                 _ => unreachable!(),
             }
         }
+        HyperType::DynamicSelective => unreachable!(),
     }
 
     let termination = config.termination.expect("no termination config");
